
    #	Process time lapse movie frames and assemble into movie

    use strict;
    use warnings;
    
    use Getopt::Long;
    
    my $version = '1.0';
    
#    my $title = "Clouds above Fourmilab, 2004-09-06";
    my $title = "The Four Seasons 2005-2006";
    my $credits = "John Walker, www.fourmilab.ch";
    my $copyright = "This movie is in the public domain";
    
    #	Show verbose processing information ?    
    my $verbose = 0;
   
    
    my $grab = '4seasons';	    # Directory containing raw frame grabs
    my $regdir = 'registered';	    # Registered frame directory
    my $regfile = 'seasons.rgd';    # Registration co-ordinate file
    
    #	Process command line options
    
    my $ook = GetOptions(
	       "copyright" => \&infoOption,
	       "help" => \&infoOption,
	       "verbose" => \$verbose,
	       "version" => \&infoOption
	       );
	       
    if (!$ook) {
    	exit(1);    	    # Command line syntax error
    }

       
    opendir(CD, $grab);
    my @frames = grep(/\.(JPG|PNG)$/i, readdir(CD));
    closedir(CD);
    @frames = sort(@frames);
    
    #	Make backup copy of rtegistration file
    command("cp -p $regfile $regfile.bak");

    my $f;
    foreach $f (@frames) {
	my $basename = $f;
	$basename =~ s/\.\w+$//;
	my $regname = "$basename.png";
	if (!(-f "$regdir/$regname")) {
	    print("Need to register $regname\n");
	    command("gimp $grab/$f &");
	    while (1) {
		print("Registration points for $basename: ");
		my $l = <>;
		if (!($l =~ m/^\s*(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s*$/)) {
	    	    print("Cannot parse registration points.\n");
		    next;
		}
		my ($x1, $y1, $x2, $y2) = ($1, $2, $3, $4);
		if ($x1 < 0 || $x1 > 3007 ||
	    	    $y1 < 0 || $y1 > 1999 ||
		    $x2 < 0 || $x2 > 3007 ||
	    	    $y2 < 0 || $y2 > 1999 ||
		    ($x1 >= $x2)) {
		    print("Registration point co-ordinates invalid.\n");
		    next;
		}
		open(RF, ">>$regfile") || die("Cannot open $regfile");
		print(RF "$basename        $x1 $y1     $x2 $y2\n");
		close(RF);
		last;
	    }
	}
    }
        
    #	Print a command and then run it
    
    sub command {
    	my ($c) = @_;
	
	print("$c\n");
	system($c);
    }
  
    #	Process information request options
    
    sub infoOption {
    	my ($opt) = @_;
	
	if ($opt eq'copyright') {
	    print("This program is in the public domain.\n");
	    exit(0);
	}
	
	if ($opt eq 'help') {
	    print << "EOD";
usage: time_lapse [ options ]

Options:
  --copyright              Display copyright information
  --help                   Display this message
  --outputfile fname       Output file name (default standard output)
  --verbose                Display verbose status information
  --version                Display version number
  
Please report bugs to bugs\@fourmilab.ch; include "time_lapse"
in the Subject line.
EOD
	    
	    exit(0);
	}
	
	if ($opt eq 'version') {
	    print("$version\n");
	    exit(0);
	}

    }
