
    #	Generate content for "Les Quatre Saisons: Jour par Jour"
    
    #	By John Walker  (http://www.fourmilab.ch/)  April 2006
    

    use strict;
    use warnings;
    use Julian qw(WEEKDAY_NAMES MONTH_ABBREVIATIONS :DEFAULT);
   
    my $regdir = '../registered';   	    # Registered image directory
    my $framedir = '../4seasons';   	    # Original frame directory
    my $calendar_months_per_row = 3;	    # Calendar months per row
    
     use constant MONTH_NAMES => [
        "January", "February", "March", "April", "May", "June",
        "July", "August", "September", "October", "November", "December" ];
   
    open(FI, "<framelist.csv") || die("Cannot open framelist.csv");
    
    my $l;
    
    $l = <FI>;	    	    # Burn title line
    
    my $i = 0;
    my (@aframeno, @aimgfile, @adtime, @aseason, @aseason_day, @adyn, @agap, @afilename);
    my (%sframe, %mframe);
    
    my %calendar;
    
    while ($l = <FI>) {
    	$l =~ m/^(\d+),"([^"]*)",([^,]*),"([^"]*)",(\d+),(?:"([^"]*)")?,"?([\d\.]*)"?,"?(\w*)"?/ || die("Cannot parse $l");
	
	my ($frameno, $imgfile, $dtime, $season, $season_day, $dyn, $gap, $filename) = ($1, $2, $3, $4, $5, $6, $7, $8);
	
	$dyn = '' if !defined($dyn);
	
    	if ($filename ne '') {
	    push(@aframeno, $frameno);
	    push(@aimgfile, $imgfile);
	    push(@adtime, $dtime);
	    push(@aseason, $season);
	    push(@aseason_day, $season_day);
	    push(@adyn, $dyn);
	    push(@agap, $gap);
	    push(@afilename, $filename);
	    
	    #	Save first frame name in season
	    if (($season ne '') && (!defined($sframe{$season}))) {
	    	$sframe{$season} = $filename;
	    }
	    
	    #	Save first frame of each month and define
	    #	calendar links.
	    if ($dtime ne '') {
	    	$dtime =~ m:^(\d+)/(\d+)/(\d+)\s:;
		my $yd = "$3-$2";
		my $ymd = sprintf("%d-%d-%d", $3, $2, $1);
		my $ym = sprintf("%d-%d", $3, $2);
		$calendar{$ymd} = "<a href=\"$filename.html\"><day></a>";
		if (!defined($mframe{$yd})) {
		    $calendar{$ym} = "<a href=\"$filename.html\"><monthyear></a>";
	    	    $mframe{$yd} = $filename;
		}
	    }
	}

    }
    close(FI);
    
    my ($imgwid, $imghgt) = (800, 549);

    for ($i = 0; $i <= $#aframeno; $i++) {
    	if ($afilename[$i] ne '') {
	
    	    #   Generate JPEG image for frame unless it's already been done
	
	    my $imtarget = "jpj/images/$afilename[$i].jpg";
	    if (!(-f $imtarget)) {
	    	my $bname = $aimgfile[$i];
		$bname =~ s/png$/jpg/;
		my $excmd = '';
	    	if (-f "$framedir/$bname") {
		    #	Extract EXIF information from original image and save
		    #	for transcription to registered image.
		    command("jpegtopnm -exif=temp.exif $framedir/$bname >/dev/null");
		    $excmd = "-exif=temp.exif";
		}
	    	command("pngtopnm $regdir/$aimgfile[$i] | pnmtojpeg $excmd -progressive -quality 75 >$imtarget");
	    }
	    
	    #	Create globe night and day view for time image was taken
	    
	    if ($adtime[$i] ne '') {
	    	$adtime[$i] =~ m-(\d+)/(\d+)/(\d+)\s+(\d+):(\d+):-;
		my ($dd, $mm, $yy, $hour, $min) = ($1, $2, $3, $4, $5);
		my $globef = "jpj/globes/$afilename[$i].jpg";
		if (!(-f $globef)) {
	    	    command('/server/src/earthview/development/earthview -i417 -l -xh50,10 -rnasa.evif -o/tmp/cache_globe.jpg -e"1' .
    	    	    	sprintf("%04d-%02d-%02d %02d:%02d", $yy, $mm, $dd, $hour - 1, $min) .
			"\" '47d4m46s' '-7d3m44s' 1000000");
		    command("mv /tmp/cache_globe.jpg $globef");
		}
	    }
	    
	    #	Get METEOSAT Weather imagery for day
	    
	    #	Copyright statement should be:
	    #	    copyright  2005 EUMETSAT
	    #	per http://www.eumetsat.int/idcplg?IdcService=SS_GET_PAGE&nodeId=417&l=en
	    
	    if ($adtime[$i] ne '') {
	    	my @mabbr = qw(X JAN FEB MAR APR MAY JUN JUL AUG SEP OCT NOV DEC);
	    	$adtime[$i] =~ m-(\d+)/(\d+)/(\d+)\s+(\d+):(\d+):-;
		my ($dd, $mm, $yy, $hour, $min) = ($1, $2, $3, $4, $5);
		my $meteof = "jpj/meteo/$afilename[$i].jpg";
		if (!(-f $meteof)) {
		    command("curl http://oiswww.eumetsat.de/webapps/ARC/SatImages/NOMINAL/$yy/" .
		    	    	$mabbr[$mm] .
				sprintf("/DAY_%02d/%d%02d%02d24VIS.jpg", $dd, $yy, $mm, $dd) .
				" >$meteof");				
		}
	    }	    
	    
	    #	Crank out HTML for page containing image
	    
	    my $title = "Les Quatre Saisons";
	    
	    my ($thisframe, $nframes) = ($i + 1, $#aframeno + 1);
	    
	    my $dt = '';
	    my ($dtev, $dtme, $iyear, $imonth, $hdate);
	    if ($adtime[$i] ne '') {
	    	$adtime[$i] =~ m-(\d+)/(\d+)/(\d+)\s+(\d+):(\d+):-;
		my ($dd, $mm, $yy, $hour, $min) = ($1, $2, $3, $4, $5);
		my $wd = WEEKDAY_NAMES->[jd_to_weekday(gregorian_to_jd($yy, $mm, $dd))];
	    	$dt = sprintf("%s %04d-%02d-%02d %02d:%02d", $wd, $yy, $mm, $dd, $hour, $min);
    	    	$dtev = sprintf("%04d-%02d-%02d %02d:%02d", $yy, $mm, $dd, $hour, $min);
    	    	$dtme = sprintf("%04d-%02d-%02d 12:00 UTC", $yy, $mm, $dd);
		$hdate = sprintf("%04d-%02d-%02d %02d%%3A%02d", $yy, $mm, $dd, $hour - 1, $min);
		$iyear = $yy;
		$imonth = $mm;
	    }
	    
	    my $dy = '';
	    if ($adyn[$i] ne '') {
	    	if ($adyn[$i] =~ m/^\-(.*)$/) {
		    $dy = "$1";
		} else {
	    	    $dy = "<b>Did you notice?</b> &nbsp; $adyn[$i].";
		}
	    }
	    
	    #	Assemble document relation links
	    my $navlinks = '<link rel="Index" href="./" />' . "\n" .
	    	    	   '<link rel="Start" href="title.html" />';
	    if ($i > 0) {
		my $lfn = $afilename[$i - 1];
	    	$navlinks .= "\n<link rel=\"Prev\" href=\"$lfn.html\" />";
	    }
	    if ($i < $#aframeno) {
		my $nfn = $afilename[$i + 1];
	    	$navlinks .= "\n<link rel=\"Next\" href=\"$nfn.html\" />";
	    }
	    
	    open(FO, ">jpj/$afilename[$i].html") || die("Cannot create jpj/$afilename[$i].html");
	    
print FO <<"EOD";
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1" />
<title>$title</title>
<link rel="stylesheet" href="../saisons.css" type="text/css" />
$navlinks
<meta name="description" content="$title" />
<meta name="author" content="John Walker" />
<meta name="keywords" content="lignieres, suisse, switzerland, timelapse, time, lapse, movie, 2005, 2006" />
<meta name="robots" content="noindex,nofollow" />
<script type="text/javascript" src="http://www.fourmilab.ch/documents/scripts/externalLinks.js">
</script>
</head>

<body onload="externalLinks();">
<table width="100%">
<tr><td class="tl">&nbsp;</td>
<td class="tm">
<h1>$title</h1>
</td>
<td class="rt">
EOD

if ($i > 0) {
    my $lfn = $afilename[$i - 1];
print FO <<"EOD";
<a class="i" rel="Prev" href="$lfn.html" title="Frame $i"><img class="b0" src="icons/prev.png" height="32" width="32"  alt="&lt;" /></a> &nbsp; &nbsp;
EOD
} else {
print FO <<"EOD";
<img class="b0" src="icons/prev_gr.png" height="32" width="32"  alt="&lt;" /> &nbsp; &nbsp;
EOD
}

print FO <<"EOD";
<a class="i" rel="Index" href="./"> <img class="b0" src="icons/up.png" height="32" width="32" alt="^" /></a> &nbsp; &nbsp;
EOD

if ($i < $#aframeno) {
    my $nfn = $afilename[$i + 1];
    my $nfnn = $i + 2;
print FO <<"EOD";
<a class="i" rel="Next" href="$nfn.html" title="Frame $nfnn"><img class="b0" src="icons/next.png" height="32" width="32" alt="&gt;" /></a>
EOD
} else {
print FO <<"EOD";
<img class="b0" src="icons/next_gr.png" height="32" width="32"  alt="&lt;" />
EOD
}

print FO <<"EOD";
</td></tr>
</table>
<hr />

<p class="c">
<img src="images/$afilename[$i].jpg" width="$imgwid" height="$imghgt" alt="" />
<br />
$dt<br />
Frame $thisframe of $nframes<br />
$dy
</p>
EOD

    if ($dt ne '') {

print FO <<"EOD";
<table class="globe">
<tr>
<td>
<a class="i" href="http://www.fourmilab.ch/cgi-bin/uncgi/Earth?img=nasa.evif&amp;imgsize=768&amp;opt=-l&amp;lat=47d4m46s&amp;ns=North&amp;lon=7d3m44s&amp;ew=East&amp;alt=1000000&amp;date=1&amp;utc=$hdate"
   rel="Target:FourmilabSaisonsAux"><img class="b0"
   src="globes/$afilename[$i].jpg" width="417" height="417" alt="Earth Viewer image for $dtev" /></a>
<br />
Day and night from<br />
<cite><a href="/earthview/" rel="Target:FourmilabSaisonsAux">Earth and Moon Viewer</a></cite></td>
<td><img class="b0" src="meteo/$afilename[$i].jpg" width="417" height="417" alt="Meteosat-7 imagery for $dtme Copyright  $iyear EUMETSAT" />
<br />
Cloud cover from Meteosat-7<br />
Copyright &copy; $iyear <a href="http://www.eumetsat.int/" rel="Target:FourmilabSaisonsAux">EUMETSAT</a></td>
</tr>
</table>
EOD
    }
    
    #	Generate season links

print FO <<"EOD";
<p class="c">
<a rel="Start" href="title.html" title="Go to title">&laquo;</a> &nbsp; &nbsp;
<a rel="Chapter" href="$sframe{Spring}.html" title="Go to first day of spring">Spring</a> &nbsp;
<a rel="Chapter" href="$sframe{Summer}.html" title="Go to first day of summer">Summer</a> &nbsp;
<a rel="Chapter" href="$sframe{Autumn}.html" title="Go to first day of autumn">Autumn</a> &nbsp;
<a rel="Chapter" href="$sframe{Winter}.html" title="Go to first day of winter">Winter</a> &nbsp;
&nbsp; <a href="credits.html" title="Go to end credits">&raquo;</a>
</p>
EOD
   
    calendar_index(\*FO, \%calendar, $calendar_months_per_row, 1);

print FO <<"EOD";

</body>
</html>
EOD
	    close(FO);
	    
    	}
    }
    
    #	Generate the calendrical index
    
    sub calendar_index {
    	my ($fh, $calendar, $calendar_months_per_row, $start_day) = @_;
	
	print($fh "<table class=\"calendar_index\">\n");
	my $inrow = 0;
	my $nrow = 0;
	
	my $lyear = -1;
    
    	my $ym;
	foreach $ym (sort(keys(%mframe))) {
    	    $ym =~ m/^(\d+)-(\d+)$/;
	    my $cyear = $1 + 0;
	    my $cmonth = $2 + 0;
	    
	    if ($lyear != $cyear) {
	    	$lyear = $cyear;
		if ($inrow) {
		    print($fh "</tr>\n");
		}
		print($fh "<tr><td class=\"y\" colspan=\"$calendar_months_per_row\"> $cyear </td></tr>\n");
	    }
	    
	    if (!$inrow) {
	    	$inrow = 1;
		$nrow = 0;
		print($fh "<tr>\n");
	    }
	    print($fh "<td>\n");
    	    calendar_month($fh, $cyear, $cmonth, \%calendar, $start_day, 1);
	    print($fh "</td>\n");
	    $nrow++;
	    if ($nrow >= $calendar_months_per_row) {
	    	print($fh "</tr>\n");
		$inrow = 0;
	    }
	}
	if ($inrow) {
	    print($fh "</tr>\n");
	}
	print($fh "</table>\n");
    }

    #	Generate a month calendar table
    
    sub calendar_month {
    	my ($fh, $year, $month, $calendar, $start_day, $pad_rows) = @_;
	
	$start_day = 0 if !defined($start_day);
	$pad_rows = 0 if !defined($pad_rows);
	$month += 0;
	
	my $jd_first = gregorian_to_jd($year, $month, 1); # Julian day of first day of month
	my $jd_last = gregorian_to_jd($year + ($month == 12 ? 1 : 0), $month == 12 ? 1 : ($month + 1), 1) - 1; # Julian day of last day of month
	
	#   Determine Julian Day of first starting  weekday before the first
	#   of the month (which can, of course, be the same day).
	my $jd = $jd_first;
	while (jd_to_weekday($jd) != $start_day) {
	    --$jd;
    	}
	
	print($fh "<table class=\"calendar_month\">\n");
	
	my $mtitle = MONTH_NAMES->[$month - 1];
	
	my $ym = sprintf("%d-%d", $year, $month);
	if (defined($calendar{$ym})) {
	    my $t = $calendar{$ym};
	    $t =~ s/<monthyear>/$mtitle/g;
	    $mtitle = $t;
	}
	
	print($fh "<tr class=\"t\"><td colspan=\"7\">" . $mtitle . "</td></tr>\n");
	
	#   Generate row of weekday labels
	print($fh "<tr class=\"d\">");
	for (my $i = 0, my $d = $start_day; $i < 7; $i++, $d = ($d + 1) % 7) {
	    print($fh " <td> " .
	    	  substr(WEEKDAY_NAMES->[$d], 0, 3) .
		  " </td> ");
	}
	print($fh "</tr>\n");
	
	#   Generate rows of calendar weeks until we have
	#   generated one which contains the last day of the
	#   month.
	
	my $n = 1;
	my $nrows = 0;
	while (1) {
	    print($fh "<tr class=\"w\">");
	    for (my $i = 0; $i < 7; $i++) {
	    	if (($jd < $jd_first) || ($jd > $jd_last)) {
		    #	Before start of after end of month: empty box
		    print($fh  ' <td class="e" /> ');
		} else {
		    my $ent = $n;
		    if (defined($calendar{"$year-$month-$n"})) {
		    	$ent = $calendar{"$year-$month-$n"};
			$ent =~ s/<day>/$n/g;
		    }
		    print($fh '<td class="d"> ' . $ent . ' </td> ');
		    $n++;
		}
		$jd++;
	    }
	    print($fh "</tr>\n");
	    $nrows++;
	    
	    if ($jd > $jd_last) {
	    	last;
	    }
	}
	
	if ($pad_rows) {
	    while ($nrows < 6) {
	    	print($fh "<tr><td class=\"d\" colspan=\"7\">&nbsp;</td></tr>\n");
		$nrows++;
	    }
	}
	
	print($fh "</table>\n");
    }

    #	Print a command and then run it
    
    sub command {
    	my ($c) = @_;
	
	print("$c\n");
	system($c);
    }
