
    #	Register images from manually-marked reference
    #	points.
    
    use strict;
    use warnings;
    
    my $imgdir = "4seasons";	    	    # Directory containing image files
    my $regdir = "Fregistered";	    	    # Directory for registered files
    my $regscript = "do_reg.sh";    	    # Shell script to re-run registration
    my $reffile = "seasons.rgd";	    # Reference point file
    my $showFileName = 1;   	    	    # Show file name in frames
    my $showDate = 1;	    	    	    # Show date and time in frames
    my $verbose = 1;	    	    	    # Verbosity level
    
    my $imgtype = "jpg";    	    	    # Image file type extension
    my $import = "jpegtopnm -quiet";	    # Command to import image to PPM
    my $export = "pnmtopng";	    	    # Command to export image
    my $exportfiletype = '.png';    	    # Export file type
    my $osizeX = 800;	    	    	    # Output image width

    open(REF, "<$reffile") || die("Cannot open reference file $reffile");
    
    my $r;
    
    $r = <REF>;     	    	# Grab first line of reference file
    $r =~ m/\s*(\w+)\s+(\d+)\s+(\d+)\s+/;   # Extract file name, first reference point
    $r = $1;
    my ($refX, $refY) = ($2, $3);
    $r = `$import $imgdir/$r.$imgtype | pamfile`;   # Extract image width and height
    $r =~ m/,\s+(\d+)\sby\s(\d+)\s/;
    my ($width, $height) = ($1, $2);	    # Original image width and height

print("Width $width height $height  RefX = $refX  RefY = $refY\n");

    seek(REF, 0, 0);
    
    my ($minX, $minY, $maxX, $maxY) = (0, 0, $width - 1, $height - 1);
    my ($meanX, $meanY, $meanN) = (0, 0, 0);
    
    #	Loop through registration commands and determine mean reference point
    
    while ($r = <REF>) {
    	if ($r =~ m/\s*(\w+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+/) {
	    my ($imfile, $ref1X, $ref1Y, $ref2X, $ref2Y) = ($1, $2, $3, $4, $5);
#	    printf("%s  Delta: %.2f\n", $imfile,
#	    	sqrt((($ref2X - $ref1X) ** 2) + (($ref2Y - $ref1Y) ** 2)));
		
	    $meanX += $ref1X;
	    $meanY += $ref1Y;
	    $meanN++;
	} else {
	    print(STDERR "Unable to parse: $r");
	}

    }
    seek(REF, 0, 0);
    
    printf("Mean alignment point: (%d, %d)  Frames: $meanN\n", int($meanX / $meanN), int($meanY / $meanN));
    $refX = int($meanX / $meanN);
    $refY = int($meanY / $meanN);
    
    #	Loop through registration commands and determine maximal registered frame
    
    while ($r = <REF>) {
    	if ($r =~ m/\s*(\w+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+/) {
	    my ($imfile, $ref1X, $ref1Y, $ref2X, $ref2Y) = ($1, $2, $3, $4, $5);

	    my ($dx, $dy) = ($refX - $ref1X, $refY - $ref1Y);
	    
	    $minX = max($minX, $dx);
	    $minY = max($minY, $dy);
	    $maxX = min($maxX, ($width + $dx) - 1);
	    $maxY = min($maxY, ($height + $dy) - 1);
	} else {
	    print(STDERR "Unable to parse: $r");
	}

    }
    seek(REF, 0, 0);
    
    print("Registered frame: ($minX, $minY) - ($maxX, $maxY)\n");
#exit(0);

    $maxX = ($maxX - $minX) + 1;
    $maxY = ($maxY - $minY) + 1;
    
    #	Create background image for compositing
    
    docmd("ppmmake rgb:00/00/00 $width $height >ref0b.ppm");
    
    open(PRO, ">$regscript") || die("Unable to create $regscript");
    
    #	Loop through registration commands and process images accordingly
    
    while ($r = <REF>) {
    	if ($r =~ m/\s*(\w+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+/) {
	    my ($imfile, $ref1X, $ref1Y, $ref2X, $ref2Y) = ($1, $2, $3, $4, $5);
	    my ($dx, $dy) = ($refX - $ref1X, $refY - $ref1Y);
	    
$showFileName = 0;	    
#if ($imfile ne 'dsc_0063') { next; }
	    
    	    my $dtime = `exif $imgdir/$imfile.$imgtype | fgrep 'Date and Time' | head -1`;
    	    if (!($dtime =~ m/\|(\d+):(\d+):(\d+)\s+(\d+):(\d+):(\d+)\s/)) {
    		$dtime = `exif $imgdir/Transformed/$imfile.$imgtype | fgrep 'Date and Time' | head -1`;
    		if (!($dtime =~ m/\|(\d+):(\d+):(\d+)\s+(\d+):(\d+):(\d+)\s/)) {
	    	    die("For transformed $imgdir/$imfile.$imgtype, cannot parse $dtime");
		}
	    }
    	    my ($dyear, $dmonth, $dmday, $dhour, $dminute, $dsecond) = ($1, $2, $3, $4, $5, $6);

    	    my $flabel = "";
	    if ($showFileName) {
	    	$flabel = "$imfile ";
	    }
	    if ($showDate) {
	    	$flabel .= "$dyear-$dmonth-$dmday $dhour:$dminute";
	    }
	    
	    #	Generate label for image
	    procmd("pbmtextps -font 'AvantGarde-Book' -fontsize 12 -resolution 720 '$flabel' | " .
	    	   "pnmscale 0.1 | " .
		   "pnmcrop -white >ref1l.ppm");
	    #	Determine size of label
	    my $imx = `pnmfile ref1l.ppm`;
	    $imx =~ m/,\s+(\d+)\sby\s(\d+)\s/ || die("Cannot obtain image label dimension for '$flabel'");
	    $imx = $1;
	    my $imy = $2;
	    #	Create solid colour image to colourise label
	    procmd("ppmmake red $imx $imy >ref1c.ppm");
	    #	Generate colour label
	    procmd("pnminvert ref1l.ppm >ref2l.ppm");
	    procmd("pamarith -multiply ref2l.ppm ref1c.ppm >ref3l.ppm");
	    
	    #	Import original image
	    procmd("$import $imgdir/$imfile.$imgtype >ref1o.ppm");
	    #	Register and scale to uniform frame size
	    procmd("pamcomp -xoff=$dx -yoff=$dy ref1o.ppm ref0b.ppm | " .
	    	   "pamcut $minX $minY $maxX $maxY | " .
	    	   "pnmscale -width $osizeX >ref1i.ppm");
#		   "ppmlabel -color rgb:FF/00/00 -size 8 -x 8 -y 16 -text '$flabel' |");
    	    #	Insert label in registered image
	    procmd("pamcomp -xoff=8 -yoff=8 -alpha=ref2l.ppm ref3l.ppm ref1i.ppm >ref2i.ppm");
	    procmd("$export ref2i.ppm >$regdir/$imfile$exportfiletype");

	} else {
	    print(STDERR "Unable to parse: $r");
	}

    }
    
    close(PRO);
    
    close(REF);
    
    #	Clean up intermediate file debris
    docmd("rm ref[0-9][a-z].ppm");

    #	Perform registration command, saving in transcript
    
    sub procmd {
    	my ($c, $vl) = @_;
	
	if (!defined($vl)) {
	    $vl = 0;
	}
	if ($vl < $verbose) {
	    print(STDERR "$c\n");
	}
	print(PRO "$c\n");
   	system($c);
    }
    
    #	Perform command without saving in transcript
    
    sub docmd {
    	my ($c, $vl) = @_;
	
	if (!defined($vl)) {
	    $vl = 0;
	}
	if ($vl < $verbose) {
	    print(STDERR "$c\n");
	}
   	system($c);
    }
    
    #	Return least of arguments

    sub min {
    	my $v = 1e307;
	my $n;
	
	foreach $n (@_) {
	    $v = $n if ($n < $v);
	}
	return $v;
    }
    
    #	Return greatest of arguments

    sub max {
    	my $v = -1e307;
	my $n;
	
	foreach $n (@_) {
	    $v = $n if ($n > $v);
	}
	return $v;
    }
